@@ -11,16 +11,24 @@
 #    under the License.
 #
 
+import argparse
 import logging
 
 from neutronclient.common import utils as n_utils
 from neutronclient.i18n import _
 from neutronclient.neutron import v2_0 as neutronV20
-from oslo.serialization import jsonutils
+from oslo_serialization import jsonutils
 
 from gbpclient.common import utils
 
 
+def _format_fixed_ips(pt):
+    try:
+        return '\n'.join([jsonutils.dumps(ip) for ip in pt['fixed_ips']])
+    except (TypeError, KeyError):
+        return ''
+
+
 def _format_network_service_params(net_svc_policy):
     try:
         return '\n'.join([jsonutils.dumps(param) for param in
@@ -42,9 +50,9 @@ class ListPolicyTarget(neutronV20.ListCommand):
 
     resource = 'policy_target'
     log = logging.getLogger(__name__ + '.ListPolicyTarget')
-    _formatters = {}
+    _formatters = {'fixed_ips': _format_fixed_ips, }
     list_columns = ['id', 'name', 'description', 'policy_target_group_id',
-                    'port_id']
+                    'port_id', 'fixed_ips']
     pagination_support = True
     sorting_support = True
 
@@ -69,6 +77,14 @@ class CreatePolicyTarget(neutronV20.CreateCommand):
         parser.add_argument(
             '--policy-target-group',
             help=_('Policy Target Group (required argument)'))
+        parser.add_argument(
+            '--fixed-ip', metavar='subnet_id=SUBNET,ip_address=IP_ADDR',
+            action='append',
+            help=_('Desired IP and/or subnet for this Policy Target: '
+                   'subnet_id=<nid>,ip_address=<ip>. '
+                   'You can repeat this option.'))
+        parser.add_argument(
+            '--fixed_ip', action='append', help=argparse.SUPPRESS)
         parser.add_argument(
             '--port-id', default='',
             help=_('Neutron Port UUID'))
@@ -91,6 +107,13 @@ class CreatePolicyTarget(neutronV20.CreateCommand):
             body[self.resource]['port_id'] = (
                 parsed_args.port_id)
 
+        ips = []
+        if parsed_args.fixed_ip:
+            for ip_spec in parsed_args.fixed_ip:
+                ip_dict = n_utils.str2dict(ip_spec)
+                ips.append(ip_dict)
+            body[self.resource]['fixed_ips'] = ips
+
         return body
 
 
@@ -114,6 +137,14 @@ class UpdatePolicyTarget(neutronV20.UpdateCommand):
         parser.add_argument(
             '--name',
             help=_('New name of the Policy Target'))
+        parser.add_argument(
+            '--fixed-ip', metavar='subnet_id=SUBNET,ip_address=IP_ADDR',
+            action='append',
+            help=_('Desired IP and/or subnet for this Policy Target: '
+                   'subnet_id=<nid>,ip_address=<ip>. '
+                   'You can repeat this option.'))
+        parser.add_argument(
+            '--fixed_ip', action='append', help=argparse.SUPPRESS)
 
     def args2body(self, parsed_args):
         body = {self.resource: {}, }
@@ -121,6 +152,13 @@ class UpdatePolicyTarget(neutronV20.UpdateCommand):
         neutronV20.update_dict(parsed_args, body[self.resource],
                                ['name', 'tenant_id', 'description'])
 
+        ips = []
+        if parsed_args.fixed_ip:
+            for ip_spec in parsed_args.fixed_ip:
+                ip_dict = n_utils.str2dict(ip_spec)
+                ips.append(ip_dict)
+            body[self.resource]['fixed_ips'] = ips
+
         return body
 
 
